  /*
*//* 
*//* H8S GPS monitor
*//*  NMEATSIP.c: NMEA and TSIP message pretty display on CLCD.
*//*     Copyright (C) 2011,2012 by HI.
*//*
*/

#include "H8S_common0.h"
#include "basicIO.h"

static BYTE debug_flag;
//
//	CLCD
//
static BYTE LCD_BUF[100];
static BYTE LCD_idx;
static volatile BYTE LCD_flg;

#define CLCD_deg	0xdf  // ''
#define CLCD_degC	"\337C" // 0xdf+'C'=

static const BYTE LCD_init[81]=
"NMEA/TSIPmonitor.    Ver1.00  by H.I.    SCI0=    _0bps 8N1 H8S_kernelYYMMDDnn_ ";
//123456789012345____0123456789012345____0123456789012345____0123456789012345____

static void CLCD_CMDw(BYTE cmd){
	CLCD_cmd(cmd);
	sys_wait_1ms(); // wait
}

static void CLCD_DATw(BYTE dat){
	CLCD_dat(dat);
	sys_wait_20us(); // wait
}

#define CLCD_setCGRAM(x) (0x40|((x)&0x3f))
#define CLCD_setDDRAM(x) (0x80|((x)&0x7f))
static void init_CLCD_CGRAM(void){ // SNRѥCGRAM򥻥å
BYTE i,c,d;
	CLCD_CMDw(CLCD_setCGRAM(0)); // set CGRAM address
	c=8;
	for(i=0;i<64;i++){
		if(0==(d=i&7))c--;
		CLCD_DATw((c<=d)?0xe:0x0); // set CG
	};
	CLCD_CMDw(CLCD_setDDRAM(0)); // set DDRAM address
};

static void init_CLCD(void){
	CLCD_init();
	sys_blkset(LCD_BUF ,' ',sizeof(LCD_BUF));
	sys_wait_10ms();  // wait for 10ms
	LCD_idx=0;	LCD_flg=0;
	sys_blkcpy(LCD_BUF,(BYTE *)LCD_init,80);
	sprintf(LCD_BUF+70,"%08X",sys_kernel_version);LCD_BUF[78]=' ';
	init_CLCD_CGRAM();   // set LCD CGRAM
	return;
}

static void LCD_display(BYTE *buf){
	switch(LCD_flg){
	  case(1):
		CLCD_cmd(CLCD_setDDRAM(0));
		LCD_idx=0; LCD_flg=0;
		break;;
	  case(2):
		CLCD_init();
		LCD_idx=0; LCD_flg=3;
		break;;
	  case(3):
		init_CLCD_CGRAM();
		LCD_flg=0;
		break;;
	  default:
		if(80>LCD_idx){
			CLCD_dat(buf[LCD_idx++]);
		}else{
			LCD_idx=0;LCD_flg=1;
		};
		break;;
	};
}


//
//	SCI
//
#define BRR(x)	((((_CPUCLOCK_)/32/((x)/10))+5)/10) - 1
const BYTE BRR_table[]={
	BRR(4800), BRR(9600), BRR(19200), BRR(38400)
#if _SCI_SPEED_== 115200
	, BRR(115200)
#endif /* _SCI_SPEED_== 115200 */
};
const INT BRR_num_table[]={
	48, 96, 192, 384
#if _SCI_SPEED_== 115200
	, 1152
#endif /* _SCI_SPEED_== 115200 */
};

static volatile BYTE SCI_flg;
static	BYTE BRR_idx;
static	INT	SCI_BR;
static	BYTE sci_rx_buf[1000];
static void init_SCI_setBR(BYTE i){
	if(sizeof(BRR_table)<=i)i=0;
	BRR_idx= i;
	SCI_BR=BRR_num_table[BRR_idx];
	sci_init(BRR_table[BRR_idx]);
	sci_set_RxBuf(sci_rx_buf,sizeof(sci_rx_buf));
	sys_wait_10ms();
	while(sci_nbuf())sci_getc();
	sys_sci_rx_flag=0;
	SCI_flg=0;
	return;
}
static void init_SCI(void){
	switch( (BTNLED()&0x03) ){
	  case(1):
		init_SCI_setBR(sizeof(BRR_table)-1);
		break;;
	  case(2):
		init_SCI_setBR(2);
		break;;
	  case(3):
		init_SCI_setBR(1);
		break;;
	  default:
		init_SCI_setBR(0);
		break;;
	};
	sprintf(LCD_BUF+46,"%4d",SCI_BR);LCD_BUF[50]='0';
	sys_NMI_dcnt8_0=255;
	while(sys_NMI_dcnt8_0){
		LCD_display(LCD_BUF);
		sleep;
	};
	sys_LED_FLAG=0xff;
	while(BTNLED()&0x03)sleep;
	while(sci_nbuf())sci_getc();
	sys_LED_FLAG=0;BTNLED();
	sys_sci_rx_flag=0;
	return;
}

//static BYTE PPS_check(void){
//	return(0!=P6.DR.BIT.B1);
//}


//
//	GPS
//
#define is_UPDATE(x)	(GPS_UPDATE_FLAG & (1<<(x)))
#define UPDATE_CLR(x)	{GPS_UPDATE_FLAG &= ~(1<<(x));}
#define UPDATE_SET(x)	{GPS_UPDATE_FLAG |=  (1<<(x));}

static LONG	GPS_UPDATE_FLAG;

#define TSIP_mask   	0xC0000000
#define NMEA_RMC_mask	0x007f0000
#define NMEA_GGA_mask	0x00001f00
#define NMEA_ZDA_mask	0x00000070
#define NMEA_GSV_mask	0x00000001

#define NMEA_GSV_SAT    	 0
#define NMEA_ZDA_hhmmss 	 4
#define NMEA_ZDA_YYYYMMDD	 5
#define NMEA_ZDA_local  	 6
#define NMEA_GGA_hhmmss 	 8
#define NMEA_GGA_LAT    	 9
#define NMEA_GGA_LON    	10
#define NMEA_GGA_ALT    	11
#define NMEA_GGA_QUALITY	12
#define NMEA_RMC_hhmmss 	16
#define NMEA_RMC_YYYYMMDD	17
#define NMEA_RMC_LAT    	18
#define NMEA_RMC_LON    	19
#define NMEA_RMC_STATUS 	20
#define NMEA_RMC_VELOCITY	21
#define NMEA_RMC_ANGLE  	22
#define TSIP_8FAB_UPDATE	30
#define TSIP_8FAC_UPDATE	31

static BYTE	NMEA_BUF[30 * 12];

struct	TSIP_8FAB_t {
	BYTE	code;	// PADDING for alignment
	BYTE	subcode;
	LONG	GPS_SecOfWeek;
	WORD	WeekNumber;
	SHORT 	UTCoffset;
	BYTE	TimingFlag;
	BYTE	ss;
	BYTE	mm;
	BYTE	hh;
	BYTE	DD;
	BYTE	MM;
	WORD	YYYY;
} __attribute__ ((packed));	// size is 20 byte.
struct	TSIP_8FAC_t {
	BYTE	subcode;
	BYTE	RecvMode;
	BYTE	DiscMode;
	BYTE	SelfSurvey;
	LONG	HoldOver;
	WORD	CriticalAlarm;
	WORD	MinorAlarm;
	BYTE	DecodeStatus;
	BYTE	DiscActivity;
	BYTE	Reserved[2];
	float	PPSoffset;
	float	OSCoffset;
	LONG	DACvalue;
	float	DACvoltage;
	float	Temperature;
	SLL 	LAT;	// psedo double (H8 arch's double type seems as float :-<)
	SLL 	LON;	// psedo double (H8 arch's double type seems as float :-<)
	SLL 	ALT;	// psedo double (H8 arch's double type seems as float :-<)
	BYTE	Reserved2[8];
} __attribute__ ((packed));	// size is 68 byte.
static union{
	struct	TSIP_8FAB_t F;
	BYTE	BUF[sizeof(struct TSIP_8FAB_t)];
} TSIP_8FAB;
static union{
	struct	TSIP_8FAC_t F;
	BYTE	BUF[sizeof(struct TSIP_8FAC_t)];
} TSIP_8FAC;


//
//	GPS message analize
//
const LONG	ctrl_mes[30]={
	(LONG)0x00000001,	// NMEA_GSV_SAT 	nn
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0007000b,	// NMEA_ZDA_hhmmss	hh:mm:ss.nnn
	(LONG)0x00090009,	// NMEA_ZDA_YYYYMMDD	ccYY/MM/DD
	(LONG)0x00040009,	// NMEA_ZDA_local  	hh:mm local
	(LONG)0x0000000b,
	(LONG)0x0007000b,	// NMEA_GGA_hhmmss	hh:mm:ss.nnn
	(LONG)0x0006000b,	// NMEA_GGA_LAT 	X nnnnn.nnnn -> Xnnn@nn.nnn'
	(LONG)0x0006000b,	// NMEA_GGA_LON 	X nnnnn.nnnn -> Xnnn@nn.nnn'
	(LONG)0x0002000a,	// NMEA_GGA_ALT 	nnn.nnnn m
	(LONG)0x00000000,	// NMEA_GGA_QUALITY	X nn
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0007000b,	// NMEA_RMC_hhmmss	hh:mm:ss.nnn
	(LONG)0x00090009,	// NMEA_RMC_YYYYMMDD	ccYY/MM/DD
	(LONG)0x0006000b,	// NMEA_RMC_LAT		X nnnnn.nnnn -> Xnnn@nn.nnn'
	(LONG)0x0006000b,	// NMEA_RMC_LON 	X nnnnn.nnnn -> Xnnn@nn.nnn'
	(LONG)0x00000002,	// NMEA_RMC_STATUS	S M
	(LONG)0x0002000a,	// NMEA_RMC_VELOCITYnnn.nnnnkt
	(LONG)0x0002000a,	// NMEA_RMC_ANGLE	nnn.nnnn @
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0000000b,
	(LONG)0x0000000b
};
static 	LONG px;

	struct editCtrl_t {
		BYTE	dummy;
		BYTE	pos;
		BYTE	bgn;
		BYTE	end;
	} __attribute__ ((packed)) p;
#if 0
BYTE *sys_memmov(BYTE *dst,BYTE *src,LONG n){
	if(n==0)return(dst);
	if(dst==src)return(dst);
	if(dst<src){
		for(;0<n;n--){
			*dst = *src;
			dst++; src++;
		};
	}else{
		for(dst+=n,src+=n;0<n;n--){
			dst--; src--;
			*dst = *src;
		};
	};
	return(dst);
}
#endif

static	LONG context_status;
struct tv_t {
	LONG	tv_int;
	LONG	tv_frac;
} __attribute__ ((packed));
struct tm_t {
	WORD	YYYY; // YYYY
	BYTE	MM; // 01-12
	BYTE	DD; // 01-28,29,30,31
	BYTE	hh; // 00-23
	BYTE	mm; // 00-59
	BYTE	ss; // 00-60
} __attribute__ ((packed));

static struct	tv_t tv;
static struct	tm_t tm;

static void	str2tm_hms(BYTE *p){
BYTE x;
	x =(*p-'0'); x=((x<<2)+x)<<1;p++;
	x+=(*p-'0'); p++;p++;
	tm.hh=x;
	x =(*p-'0'); x=((x<<2)+x)<<1;p++;
	x+=(*p-'0'); p++;p++;
	tm.mm=x;
	x =(*p-'0'); x=((x<<2)+x)<<1;p++;
	x+=(*p-'0'); p++;p++;
	tm.ss=x;
}
static void	str2tm_YMD(BYTE *p){
WORD x;
	x =(*p-'0'); x=((x<<2)+x)<<1;p++;
	x+=(*p-'0'); x=((x<<2)+x)<<1;p++;
	x+=(*p-'0'); x=((x<<2)+x)<<1;p++;
	x+=(*p-'0'); p++;p++;
	tm.YYYY=x;
	x =(*p-'0'); x=((x<<2)+x)<<1;p++;
	x+=(*p-'0'); p++;p++;
	tm.MM=x;
	x =(*p-'0'); x=((x<<2)+x)<<1;p++;
	x+=(*p-'0'); p++;p++;
	tm.DD=x;
}

static BYTE *GPSmesEdit(BYTE *buf,struct editCtrl_t *ctrl,const BYTE statMSB,const BYTE c){
	if(statMSB & 0x80){
		if(0x80==statMSB){
			(ctrl->pos)++;
			if(ctrl->end >= ctrl->pos) buf[ctrl->pos]=c;
		}else{
			if(ctrl->end >= (ctrl->pos + (statMSB & 0x7f)) ) buf[ctrl->pos + (statMSB & 0x7f)]=c;
		};
	}else{
		if(buf[ctrl->bgn]>' '){
			(ctrl->pos)++;
		}else{
			sys_memmove(buf+ctrl->bgn, buf+ctrl->bgn+1, ctrl->pos - ctrl->bgn);
		};
		if(ctrl->end >= ctrl->pos) buf[ctrl->pos]=c;
	};
	return(buf);
}

#define if_NMEA_reset_item(x)	\
	if(0==i32.b.b3){ \
		px=ctrl_mes[(x)]; \
		sys_blkset(NMEA_BUF+12*(x),' ',12); \
		UPDATE_CLR(x); \
	}
static BYTE	NMEA_GSV_SNR[40],*NMEA_GSV_SNRp;
static BYTE NMEA_GSV_total,NMEA_GSV_seqNO;
static void NMEA(BYTE c){
	switch(i32.b.b1){	// NMEAΥƥ󥹥פǽ򿶤ʬ
	  case(0x01):	// GGA
		switch(i32.b.b2){	// NMEAΥƥ󥹥եɤȤ˽򿶤ʬ
		  case(1): // UTC hhmmss
//			if(0==i32.b.b3){
//				printf("GGA\r\n");
//			};
			if_NMEA_reset_item(NMEA_GGA_hhmmss)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_GGA_hhmmss,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				switch(i32.b.b3){
				  case(2):
				  case(4):
					GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_GGA_hhmmss,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,':');
					break;;
				  case(6):
					UPDATE_SET(NMEA_GGA_hhmmss);
					break;;
				  default:
					break;;
				};
			};
			break;;
		  case(2): // LAT
			if_NMEA_reset_item(NMEA_GGA_LAT)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_GGA_LAT,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
			};
			break;;
		  case(3): // LAT N/S
			if(1==i32.b.b3){
				sys_memmove(NMEA_BUF+12*NMEA_GGA_LAT,NMEA_BUF+12*NMEA_GGA_LAT+1,4);
				NMEA_BUF[12*NMEA_GGA_LAT]=c;
				NMEA_BUF[12*NMEA_GGA_LAT+4]=CLCD_deg;
				UPDATE_SET(NMEA_GGA_LAT);
			};
			break;;
		  case(4): // LON
			if_NMEA_reset_item(NMEA_GGA_LON)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_GGA_LON,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
			};
			break;;
		  case(5): // LON E/W
			if(1==i32.b.b3){
				sys_memmove(NMEA_BUF+12*NMEA_GGA_LON,NMEA_BUF+12*NMEA_GGA_LON+1,4);
				NMEA_BUF[12*NMEA_GGA_LON]=c;
				NMEA_BUF[12*NMEA_GGA_LON+4]=CLCD_deg;
				UPDATE_SET(NMEA_GGA_LON);
			};
			break;;
		  case(6): // GPSquality
			if_NMEA_reset_item(NMEA_GGA_QUALITY)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_GGA_QUALITY,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				UPDATE_SET(NMEA_GGA_QUALITY);
			};
			break;;
		  case(7): // GPS number of satellaites
			switch(i32.b.b3){
			  case(1):
			  case(2):
				NMEA_BUF[12*NMEA_GGA_QUALITY+i32.b.b3+1]=c;
				break;;
			  default:
				break;;
			};
			break;;
		  case(9): // ALT
			if_NMEA_reset_item(NMEA_GGA_ALT)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_GGA_ALT,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				if(0x80==i32.b.b3)UPDATE_SET(NMEA_GGA_ALT);
			};
			break;;
		  default:
			break;;
		};
		break;;
	  case(0x02):	// RMC
		switch(i32.b.b2){	// NMEAΥƥ󥹥եɤȤ˽򿶤ʬ
		  case(1): // UTC hhmmss
//			if(0==i32.b.b3){
//				printf("RMC\r\n");
//			};
			if_NMEA_reset_item(NMEA_RMC_hhmmss)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_hhmmss,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				switch(i32.b.b3){
				  case(2):
				  case(4):
					GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_hhmmss,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,':');
					break;;
				  case(6):
					UPDATE_SET(NMEA_RMC_hhmmss);
					break;;
				  default:
					break;;
				};
			};
			break;;
		  case(2): // STATUS
			if_NMEA_reset_item(NMEA_RMC_STATUS)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_STATUS,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				UPDATE_SET(NMEA_RMC_STATUS);
			};
			break;;
		  case(3): // LAT
			if_NMEA_reset_item(NMEA_RMC_LAT)
			else	GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_LAT,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
			break;;
		  case(4): // LAT N/S
			if(1==i32.b.b3){
				sys_memmove(NMEA_BUF+12*NMEA_RMC_LAT,NMEA_BUF+12*NMEA_RMC_LAT+1,4);
				NMEA_BUF[12*NMEA_RMC_LAT]=c;
				NMEA_BUF[12*NMEA_RMC_LAT+4]=CLCD_deg;
				UPDATE_SET(NMEA_RMC_LAT);
			};
			break;;
		  case(5): // LON
			if_NMEA_reset_item(NMEA_RMC_LON)
			else	GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_LON,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
			break;;
		  case(6): // LON E/W
			if(1==i32.b.b3){
				sys_memmove(NMEA_BUF+12*NMEA_RMC_LON,NMEA_BUF+12*NMEA_RMC_LON+1,4);
				NMEA_BUF[12*NMEA_RMC_LON]=c;
				NMEA_BUF[12*NMEA_RMC_LON+4]=CLCD_deg;
				UPDATE_SET(NMEA_RMC_LON);
			};
			break;;
		  case(7): // VELOCITY
			if_NMEA_reset_item(NMEA_RMC_VELOCITY)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_VELOCITY,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				if(0x80==i32.b.b3)UPDATE_SET(NMEA_RMC_VELOCITY);
			};
			break;;
		  case(8): // DIRECTION ANGLE
			if_NMEA_reset_item(NMEA_RMC_ANGLE)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_ANGLE,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				if(0x80==i32.b.b3)UPDATE_SET(NMEA_RMC_ANGLE);
			};
			break;;
		  case(9): // DDMMYY
			if_NMEA_reset_item(NMEA_RMC_YYYYMMDD)
			else {
				if(1==i32.b.b3||3==i32.b.b3){
					GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_YYYYMMDD,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,'/');
				};
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_RMC_YYYYMMDD,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				switch(i32.b.b3){
				  case(2):
				  case(4):
					px=(((px>>16)-3)<<16)|(px&0xffff);
					break;;
				  case(6):
					NMEA_BUF[12*NMEA_RMC_YYYYMMDD]='2';
					NMEA_BUF[12*NMEA_RMC_YYYYMMDD+1]='0';
#if 0
					str2tm_hms(NMEA_BUF+12*NMEA_RMC_hhmmss);
					str2tm_YMD(NMEA_BUF+12*NMEA_RMC_YYYYMMDD);
					timeutl_tm2tv(&tm, &tv);
					sys_time_sec= tv.tv_int; // set systime
#endif
					UPDATE_SET(NMEA_RMC_YYYYMMDD);
					break;;
				  default:
					break;;
				};
			};
			break;;
		  case(12): // MODE
			NMEA_BUF[12*NMEA_RMC_STATUS+2]=c;
			UPDATE_SET(NMEA_RMC_STATUS);
			break;;
		  default:
			break;;
		};
		break;;
	  case(0x03):	// ZDA
		switch(i32.b.b2){	// NMEAΥƥ󥹥եɤȤ˽򿶤ʬ
		  case(1): // UTC hhmmss
//			if(0==i32.b.b3){
//				printf("ZDA\r\n");
//			};
			if_NMEA_reset_item(NMEA_ZDA_hhmmss)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_ZDA_hhmmss,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				switch(i32.b.b3){
				  case(2):
				  case(4):
					GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_ZDA_hhmmss,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,':');
					break;;
				  case(6):
					UPDATE_SET(NMEA_ZDA_hhmmss);
					break;;
				  default:
					break;;
				};
			};
			break;;
		  case(2): // UTC DD
			if_NMEA_reset_item(NMEA_ZDA_YYYYMMDD);
		  case(3): // UTC MM
			if(0!=i32.b.b3){
				if(1==i32.b.b3){
					GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_ZDA_YYYYMMDD,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,'/');
				};
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_ZDA_YYYYMMDD,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				if(2==i32.b.b3){
					px=(((px>>16)-3)<<16)|(px&0xffff);
				};
			};
			break;;
		  case(4): // UTC YYYY
			if(0!=i32.b.b3){
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_ZDA_YYYYMMDD,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				if(4==i32.b.b3){
					str2tm_YMD(NMEA_BUF+12*NMEA_ZDA_YYYYMMDD);
					str2tm_hms(NMEA_BUF+12*NMEA_ZDA_hhmmss);
					timeutl_tm2tv(&tm, &tv);
					sys_time_sec= tv.tv_int; // set systime
					UPDATE_SET(NMEA_ZDA_YYYYMMDD);
				};
			};
			break;;
		  case(5): // local hh
			if_NMEA_reset_item(NMEA_ZDA_local)
			else{
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_ZDA_local,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				if(2==i32.b.b3){
					GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_ZDA_local,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,':');
				};
			};
			break;;
		  case(6): // local mm
			if(0!=i32.b.b3){
				GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_ZDA_local,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
				if(2==i32.b.b3){
					sys_blkcpy((BYTE *)NMEA_BUF+12*NMEA_ZDA_local+6,"local",5);
					UPDATE_SET(NMEA_ZDA_local);
				};
			};
		  default:
			break;;
		};
		break;;
	  case(0x04):	// GSV
		switch(i32.b.b2){	// NMEAΥƥ󥹥եɤȤ˽򿶤ʬ
		  case(1):
			NMEA_GSV_total= c - '0';
//			if(0==i32.b.b3){
//				printf("GSV\r\n");
//			};
			break;;
		  case(2): // GSV sequence number
			NMEA_GSV_seqNO= c - '0';
			if(1==NMEA_GSV_seqNO){
				sys_blkset(NMEA_GSV_SNRp+(NMEA_GSV_total<<2),'x',sizeof(NMEA_GSV_SNR)-(NMEA_GSV_total<<2));
				NMEA_GSV_SNR[sizeof(NMEA_GSV_SNRp)-1]=0;
			};
			break;;
		  case(3): // satellite availabel
			if(1==NMEA_GSV_seqNO){
				if_NMEA_reset_item(NMEA_GSV_SAT)
				else{
					GPSmesEdit((BYTE *)NMEA_BUF+12*NMEA_GSV_SAT,(struct editCtrl_t *) &px,(const BYTE) i32.b.b3,(const BYTE)c);
					if(4==i32.b.b3){
						UPDATE_SET(NMEA_GSV_SAT);
					};
				};
			};
			break;;
		  case(7): // SNR
		  case(11):
		  case(15):
		  case(19):
			switch(i32.b.b3){
			  case(0):
				NMEA_GSV_SNRp[((NMEA_GSV_seqNO - 1)<<2) + ((i32.b.b2 - 7)>>2)]='x';
				break;;
			  case(1):
				NMEA_GSV_SNRp[((NMEA_GSV_seqNO - 1)<<2) + ((i32.b.b2 - 7)>>2)]=((c-'0')>6)?7:(c-'0');
				break;;
			  default:
				break;;
			};
			break;;
		  default:
			break;;
		};
		break;;
	  default:
		break;;
	};
}

static void TSIP(BYTE c){
	switch(i32.b.b1){
	  case(0xAB):	// 8FAB
		if(17>i32.w.w1){
			switch(i32.w.w1){
			  case(0):
				UPDATE_CLR(TSIP_8FAB_UPDATE);
				break;;
			  case(16):
				TSIP_8FAB.BUF[0]=0x8F;
				UPDATE_SET(TSIP_8FAB_UPDATE);
				break;;
			  default:
				break;;
			};
			TSIP_8FAB.BUF[i32.w.w1 + 1]=c;
		};
		break;;
	  case(0xAC):	// 8FAC
		if(60>i32.w.w1){
			switch(i32.w.w1){
			  case(0):
				UPDATE_CLR(TSIP_8FAC_UPDATE);
				break;;
			  case(59):
				muldiv_dbxul((void *)&(TSIP_8FAC.F.LAT),180UL*113UL, 355UL); // rad2deg
				muldiv_dbxul((void *)&(TSIP_8FAC.F.LON),180UL*113UL, 355UL); // rad2deg
				UPDATE_SET(TSIP_8FAC_UPDATE);
				if(debug_flag&1){
					TSIP_8FAC.F.MinorAlarm=0xffff;
				};
				if(debug_flag&2){
					TSIP_8FAC.F.CriticalAlarm=0xffff;
				};
				break;;
			  default:
				break;;
			};
			TSIP_8FAC.BUF[i32.w.w1]=c;
		};
		break;;
	  default:
		break;;
	};
}

static void GPSmessage(BYTE c){
	i32.l= context_status= GPScontext(context_status, c);	// GPSåʸ̮ϥơȥޥʸȤ˸ƤӽФ
// 
	switch(i32.b.b0){	// NMEAޤTSIPǳꤷ򿶤ʬ
	  case(0x0f):
		NMEA(c);
		sys_NMI_dcnt8_0=255;
		break;;
	  case(0x14):
		TSIP(c);
		sys_NMI_dcnt8_1=255;
		break;;
	  default:
		break;;
	};
}

//
// ɽԽ䤽ʤ
//
static	volatile	BYTE	LCD_st0,LCD_st1;
static	LONG	TVsec;
static	INT 	TZoffset;

static volatile enum {mode_NONE,mode_NMEA,mode_TSIP} mode;
static const BYTE *str_week[]={
	"Sun","Mon","Tue","Wed","Thu","Fri","Sat","***"
};
static const struct tzctrl_t{
	BYTE *str_tzname;
	INT  tzvalue;
} tz_table[]= {
	{ "UTC",0 },
	{ "JST",3600*9 }
};
static BYTE *tznm;

static void	edit_NMEA_time(void){
BYTE *s1,*s2,*s3;
	if(LCD_st0 & 1){
		TZoffset= tz_table[1].tzvalue;
		tznm= tz_table[1].str_tzname;
	}else{
		TZoffset= tz_table[0].tzvalue;
		tznm= tz_table[0].str_tzname;
	};
	s1= s2= "****"; s3="**";
	if(is_UPDATE(NMEA_GGA_QUALITY)){
		switch(NMEA_BUF[12*NMEA_GGA_QUALITY]){
		  case('0'):
			s1= " N/A";
			break;;
		  case('1'):
			s1= "SNGL";
			break;;
		  case('2'):
			s1= "DGPS";
			break;;
		  default:
			break;;
		};
	};
	if(is_UPDATE(NMEA_RMC_STATUS)){
		switch(NMEA_BUF[12*NMEA_RMC_STATUS+2]){
		  case('N'):
			s2= "N/A ";
			break;;
		  case('A'):
			s2= "AUTO";
			break;;
		  case('D'):
			s2= "DGPS";
			break;;
		  default:
			break;;
		};
		switch(NMEA_BUF[12*NMEA_RMC_STATUS]){
		  case('A'):
			s3= "OK";
			break;;
		  case('V'):
			s3= "NG";
			break;;
		  default:
			break;;
		};
	};

	sys_time_gettime(&tv);
	tv.tv_int+=TZoffset;
	timeutl_tv2tm(&tm, &tv);
	sprintf(LCD_BUF   ,"%02d:%02d:%02d %2d/%2d %s %c %s:%c%c,%s-%s   ",
		tm.hh, tm.mm, tm.ss, tm.MM, tm.DD, tznm, ((sys_sci_rx_flag)?'!':' '),
		s1, NMEA_BUF[12*NMEA_GGA_QUALITY+2],NMEA_BUF[12*NMEA_GGA_QUALITY+3], s3, s2
	);
	sys_blkset(LCD_BUF+39, ' ', 41);
	switch(LCD_st1 & 3){
	  case(1):
		sys_blkcpy(LCD_BUF+40, "LAT ", 4);
		if(is_UPDATE(NMEA_GGA_LAT)){
			sys_blkcpy(LCD_BUF+44, NMEA_BUF+12*NMEA_GGA_LAT, 12);
			sys_blkcpy(LCD_BUF+56, "'GGA", 4);
		}else{
			if(is_UPDATE(NMEA_RMC_LAT)){
				sys_blkcpy(LCD_BUF+44, NMEA_BUF+12*NMEA_RMC_LAT, 12);
				sys_blkcpy(LCD_BUF+56, "'RMC", 4);
			};
		};
		break;;
	  case(2):
		sys_blkcpy(LCD_BUF+40, "LON ", 4);
		if(is_UPDATE(NMEA_GGA_LON)){
			sys_blkcpy(LCD_BUF+44, NMEA_BUF+12*NMEA_GGA_LON, 12);
			sys_blkcpy(LCD_BUF+56, "'GGA", 4);
		}else{
			if(is_UPDATE(NMEA_RMC_LON)){
				sys_blkcpy(LCD_BUF+44, NMEA_BUF+12*NMEA_RMC_LON, 12);
				sys_blkcpy(LCD_BUF+56, "'RMC", 4);
			};
		};
		break;;
	  case(3):
		sys_blkcpy(LCD_BUF+40, "ALT ", 4);
		if(is_UPDATE(NMEA_GGA_ALT)){
			sys_blkcpy(LCD_BUF+44, NMEA_BUF+12*NMEA_GGA_ALT, 12);
			sys_blkcpy(LCD_BUF+54, "m     ", 6);
		};
		break;;
	  default:
		sprintf(LCD_BUF+40,"SNR=");
		sys_blkcpy(LCD_BUF+44, NMEA_GSV_SNRp, 32);
		break;;
	};
	sprintf(LCD_BUF+60," %04d/%02d/%02d (%s)  ", tm.YYYY, tm.MM, tm.DD, str_week[timeutl_day_of_week(&tm) & 7]);
	LCD_BUF[79]=' ';
	mode=mode_NMEA;
	sys_LED_FLAG=0x00;
}

const BYTE *DMODE[]={
	"Normal ",
	"PowerUp",
	"AHoldOV",
	"MHoldOV",
	"Recover",
	"NotUsed",
	"Disable",
	"*******"
};
const BYTE *DACTY[]={
	"Ph Lock",
	"OSCwarm",
	"FrqLock",
	"PlacPPS",
	"IniLoop",
	"CompOSC",
	"Inactiv",
	"NotUsed",
	"Recover",
	"*******"
};
const BYTE *WarnMES[]={
	"Voltage NearRail",	// MinorWarning bit0
	"Antenna Open    ",
	"Antenna Shorted ",
	"NotTrackingSats.",
	"NotDisciplin OSC",
	"Survey-in Progre",
	"NoStoredPosition",
	"LeapSecondPendin",
	"In test mode    ",
	"PositionQuestion",
	"EEPROM Invalid  ",
	"Almanac Invalid ",
	"  MINOR bit12 ON",
	"  MINOR bit13 ON",
	"  MINOR bit14 ON",
	"  MINOR bit15 ON",	// MinorWarning bit15
	"ROM CheckSUM Err",	// Critical Warning bit0
	"RAM Check Failed",
	"PowerSupply Fail",
	"FPGA CheckFailed",
	"OSC Voltage@Rail",
	"CRITICAL bit5 ON",
	"CRITICAL bit6 ON",
	"CRITICAL bit7 ON",
	"CRITICAL bit8 ON",
	"CRITICAL bit9 ON",
	"CRITICAL bit10ON",
	"CRITICAL bit11ON",
	"CRITICAL bit12ON",
	"CRITICAL bit13ON",
	"CRITICAL bit14ON",
	"CRITICAL bit15ON"	// Critical Warning bit15
};

static	BYTE WarnMES_idx,Warn_FLAG;
static	BYTE fnum_buf[16];
static	BYTE *edit_fnum(BYTE *buf,INT i,SHORT e){
	sys_blkset(buf, ' ', 12);
	sprintf(buf,"%09d",i);
	if('-' != *buf){
		sys_memmove(buf+1,buf,10);
		*buf=' ';
	};
	switch(e){
	  case(-3):
		sys_memmove(buf+6,buf+1,6);
		sys_blkset(buf+1,'0',5);
		buf[2]='.';
		break;;
	  case(-2):
		sys_memmove(buf+5,buf+1,7);
		sys_blkset(buf+1,'0',4);
		buf[2]='.';
		break;;
	  case(-1):
		sys_memmove(buf+4,buf+1,8);
		buf[1]='0';
		buf[2]='.';
		buf[3]='0';
		break;;
	  case(0):
		sys_memmove(buf+3,buf+1,9);
		buf[1]='0';
		buf[2]='.';
		break;;
	  case(1):
		sys_memmove(buf+3,buf+2,8);
		buf[2]='.';
		break;;
	  case(2):
		sys_memmove(buf+4,buf+3,7);
		buf[3]='.';
		break;;
	  case(3):
		sys_memmove(buf+5,buf+4,6);
		buf[4]='.';
		break;;
	  case(4):
		sys_memmove(buf+6,buf+5,5);
		buf[5]='.';
		break;;
	  case(5):
		sys_memmove(buf+7,buf+6,4);
		buf[6]='.';
		break;;
	  default:
		sys_memmove(buf+3,buf+2,8);
		buf[2]='.';
		if(e>5){
			sprintf(buf+6,"E%02d",e-1);
		}else{
			sprintf(buf+5,"E%03d",e-1);
		};
		break;;
	};
	return(buf);
}

static	INT 	expint_int;
static	SHORT	expint_exp;
static void	edit_TSIP_mes(BYTE st){
	switch(st & 7){
	  case(1):
		fl2expint(&expint_int,&expint_exp,&(TSIP_8FAC.F.PPSoffset));
		sys_memcpy(LCD_BUF+40,"PPS",3);
		sys_blkcpy(LCD_BUF+44, edit_fnum(fnum_buf,expint_int,expint_exp), 9);
		sys_memcpy(LCD_BUF+53,"ns",2);
		break;;
	  case(2):
		fl2expint(&expint_int,&expint_exp,&(TSIP_8FAC.F.OSCoffset));
		sys_memcpy(LCD_BUF+40,"10M",3);
		sys_blkcpy(LCD_BUF+44, edit_fnum(fnum_buf,expint_int,expint_exp), 9);
		sys_memcpy(LCD_BUF+53,"ppb",3);
		break;;
	  case(3):
		fl2expint(&expint_int,&expint_exp,&(TSIP_8FAC.F.Temperature));
		sys_memcpy(LCD_BUF+40,"TEMP",4);
		sys_blkcpy(LCD_BUF+44, edit_fnum(fnum_buf,expint_int,expint_exp), 9);
		sys_memcpy(LCD_BUF+53, CLCD_degC,2);
		break;;
	  case(4):
		fl2expint(&expint_int,&expint_exp,&(TSIP_8FAC.F.DACvoltage));
		sys_memcpy(LCD_BUF+40,"DAC",3);
		sys_blkcpy(LCD_BUF+44, edit_fnum(fnum_buf,expint_int,expint_exp), 9);
		sys_memcpy(LCD_BUF+53, "V", 1);
		sprintf(LCD_BUF+60,"value=0x%08X",TSIP_8FAC.F.DACvalue);
		LCD_BUF[76]=' ';
		break;;
	  case(5):
		db2expint(&expint_int,&expint_exp,(void *)&(TSIP_8FAC.F.LAT));
		sys_memcpy(LCD_BUF+40,"LAT",3);
		sys_blkcpy(LCD_BUF+44, edit_fnum(fnum_buf,expint_int,expint_exp), 10);
		LCD_BUF[44]=('-'==LCD_BUF[44])?'S':'N';
		LCD_BUF[54]=CLCD_deg;
		sys_memcpy(LCD_BUF+56,"deg LON",7);
		db2expint(&expint_int,&expint_exp,(void *)&(TSIP_8FAC.F.LON));
		sys_blkcpy(LCD_BUF+64, edit_fnum(fnum_buf,expint_int,expint_exp), 10);
		LCD_BUF[64]=('-'==LCD_BUF[64])?'W':'E';
		LCD_BUF[74]=CLCD_deg;
		sys_memcpy(LCD_BUF+76,"deg ",7);
		break;;
	  case(6):
		db2expint(&expint_int,&expint_exp,(void *)&(TSIP_8FAC.F.LON));
		sys_memcpy(LCD_BUF+40,"LON",3);
		sys_blkcpy(LCD_BUF+44, edit_fnum(fnum_buf,expint_int,expint_exp), 10);
		LCD_BUF[44]=('-'==LCD_BUF[44])?'W':'E';
		LCD_BUF[54]=CLCD_deg;
		sys_memcpy(LCD_BUF+56,"deg ALT",7);
		db2expint(&expint_int,&expint_exp,(void *)&(TSIP_8FAC.F.ALT));
		sys_blkcpy(LCD_BUF+64, edit_fnum(fnum_buf,expint_int,expint_exp), 10);
		LCD_BUF[74]='m';
		break;;
	  case(7):
		db2expint(&expint_int,&expint_exp,(void *)&(TSIP_8FAC.F.ALT));
		sys_memcpy(LCD_BUF+40,"ALT",3);
		sys_blkcpy(LCD_BUF+44, edit_fnum(fnum_buf,expint_int,expint_exp), 10);
		sys_memcpy(LCD_BUF+54,"m     LAT",9);
		db2expint(&expint_int,&expint_exp,(void *)&(TSIP_8FAC.F.LAT));
		sys_blkcpy(LCD_BUF+64, edit_fnum(fnum_buf,expint_int,expint_exp), 10);
		LCD_BUF[64]=('-'==LCD_BUF[64])?'S':'N';
		LCD_BUF[74]=CLCD_deg;
		sys_memcpy(LCD_BUF+76,"deg ",7);
		break;;
	  default:
		sprintf(LCD_BUF+40,"%s,%s",
			DMODE[TSIP_8FAC.F.DiscMode], DACTY[TSIP_8FAC.F.DiscActivity]
		);
		LCD_BUF[55]='.';
		break;;
	};
}

static void	edit_TSIP_time(void){
	if(is_UPDATE(TSIP_8FAB_UPDATE)){	UPDATE_CLR(TSIP_8FAB_UPDATE);
		tm.YYYY= TSIP_8FAB.F.YYYY;
		tm.MM= TSIP_8FAB.F.MM;
		tm.DD= TSIP_8FAB.F.DD;
		tm.hh= TSIP_8FAB.F.hh;
		tm.mm= TSIP_8FAB.F.mm;
		tm.ss= TSIP_8FAB.F.ss;
		timeutl_tm2tv(&tm, &tv);
		if(0==(TSIP_8FAB.F.TimingFlag & 1)){
			tv.tv_int-=TSIP_8FAB.F.UTCoffset;
		};
		sys_time_sec= tv.tv_int; // set systime
		if(LCD_st0 & 1){
			TZoffset= tz_table[1].tzvalue;
			tznm= tz_table[1].str_tzname;
		}else{
			TZoffset= tz_table[0].tzvalue;
			tznm= tz_table[0].str_tzname;
		};
		tv.tv_int+=TZoffset;
		timeutl_tv2tm(&tm, &tv);
		sprintf(LCD_BUF   ,"%02d:%02d:%02d %2d-%2d %3s %c%04d  offset=%3d(s)",
			tm.hh, tm.mm, tm.ss, tm.MM, tm.DD,
			tznm, (sys_sci_rx_flag)?'!':' ' , tm.YYYY, TSIP_8FAB.F.UTCoffset
		);
		LCD_BUF[39]=' ';
	};
	if(is_UPDATE(TSIP_8FAC_UPDATE)){	UPDATE_CLR(TSIP_8FAC_UPDATE);
		if((debug_flag&0x80)||(0==(TSIP_8FAC.F.CriticalAlarm|TSIP_8FAC.F.MinorAlarm))){
			sys_blkset(LCD_BUF+40, ' ', 40);
			edit_TSIP_mes(LCD_st1);
			if(TSIP_8FAC.F.DiscMode){
				sys_LED_FLAG^=0x04;
			}else{
				sys_LED_FLAG=0x04;
			};
			if(debug_flag&0x80)LCD_BUF[18]='.';
		}else{
			if(0==(tv.tv_int&2)){
				if(0==Warn_FLAG)Warn_FLAG=1;
			}else{
				if(Warn_FLAG){Warn_FLAG=0;
					sys_blkset(LCD_BUF+40, ' ', 40);
					WarnMES_idx++;
					while(16>WarnMES_idx){
						if(TSIP_8FAC.F.MinorAlarm&(1<<WarnMES_idx))break;;
						WarnMES_idx++;
					};
					if(16<=WarnMES_idx){
						while(32>WarnMES_idx){
							if(TSIP_8FAC.F.CriticalAlarm&(1<<(WarnMES_idx-16)))break;;
							WarnMES_idx++;
						};
						if(32<=WarnMES_idx){
							WarnMES_idx=0;
							edit_TSIP_mes(0);
						}else{
							sys_blkcpy(LCD_BUF+40,WarnMES[WarnMES_idx],16);
						};
					}else{
						sys_blkcpy(LCD_BUF+40,WarnMES[WarnMES_idx],16);
					};
					sprintf(LCD_BUF+56,"%3d%%%04X-%04X%7dsec",
						TSIP_8FAC.F.SelfSurvey,
						TSIP_8FAC.F.CriticalAlarm, TSIP_8FAC.F.MinorAlarm,
						TSIP_8FAC.F.HoldOver
					);
				};
			};
			sys_LED_FLAG=0x08;
		};
	};
	mode=mode_TSIP;
}

static void	edit_sys_time(void){
	sys_time_gettime(&tv);
	if(TVsec==tv.tv_int)	return;
	TVsec=tv.tv_int;
	if(LCD_st0 & 1){
		TZoffset= tz_table[1].tzvalue;
		tznm= tz_table[1].str_tzname;
	}else{
		TZoffset= tz_table[0].tzvalue;
		tznm= tz_table[0].str_tzname;
	};
	tv.tv_int+=TZoffset;
	timeutl_tv2tm(&tm, &tv);
	sprintf(LCD_BUF   ,"%02d:%02d:%02d %2d.%2d %s        %4d00bps. 8N1",
		tm.hh, tm.mm, tm.ss, tm.MM, tm.DD, tznm, SCI_BR
	);
	sprintf(LCD_BUF+40,"%04d/%02d/%02d (%s)    NMEA/TSIPmonitor.  ",
		tm.YYYY, tm.MM, tm.DD, str_week[timeutl_day_of_week(&tm) & 7]
	);
	LCD_BUF[79]=' ';
	mode=mode_NONE;
	sys_LED_FLAG=0x0c;
};

static LONG BTNtime;
void intrruption(void){
BYTE c;
	c=BTNLED();
	if((0x11==(0x11&c))){  	// Aܥ󤬲줿
		if(mode==mode_NONE){
			SCI_flg=2;	// SCI󥹳ϻؼ
		}else{
			LCD_st1++;
		};
	};
	if((0x22==(0x22&c))){  	// Bܥ󤬲줿
		LCD_st0++;
		LCD_flg=2;	// CLCD󥹳ϻؼ
		sys_sci_rx_flag=0;	// SCI顼ե饰ꥢ
	};
#if 0
	if((0x23==(0x33&c))){  	// Aܥ󤬲줿ޤޤBܥ򲡤줿
		debug_flag++;
	};
	if((0x13==(0x33&c))){  	// Bܥ󤬲줿ޤޤAܥ򲡤줿
		debug_flag=0;
	};
#endif
	if((0x01==(0x03&c))){  	// Aܥ󤬲줿ޤ޾
		BTNtime++;
	};
	if((BTNtime>500)){  	// Aܥ󤬲줿ޤĹ֤ξ
		BTNtime=0;
		debug_flag^=0x80;
	};
	if((0x01!=(0x03&c))){  	// Aܥ󤬲줿
		BTNtime=0;
	};
}

void NMEA_TSIP_monitor(void){
	debug_flag=0;
	init_CLCD();    // LCD initialize
	init_SCI();
	tm.YYYY=2012; tm.MM=1; tm.DD=1;
	tm.hh=0; tm.mm=0; tm.ss=0;
	timeutl_tm2tv(&tm, &tv);
	sys_time_settime(&tv);
	EI;

	mode=mode_NONE;
	TZoffset=0;
	LCD_st0=LCD_st1=0;
	WarnMES_idx=0;Warn_FLAG=0;

	NMEA_GSV_SNRp= NMEA_GSV_SNR;
	GPS_UPDATE_FLAG= 0;

	while(1){
		if(0==sys_NMI_dcnt16){sys_NMI_dcnt16=3;
			intrruption();
			if(SCI_flg){
				switch(BRR_idx){
				  case(0):
					init_SCI_setBR(1);
					break;;
				  case(1):
					init_SCI_setBR(2);
					break;;
				  case(2):
					init_SCI_setBR(3);
					break;;
	#if _SCI_SPEED_== 115200
				  case(3):
					init_SCI_setBR(4);
					break;;
	#endif /* _SCI_SPEED_== 115200 */
				  default:
					init_SCI_setBR(0);
					break;;
				};
			};
		};
		LCD_display(LCD_BUF);
		sleep;
		while(sci_nbuf())GPSmessage(sci_getc());
		if(sys_NMI_dcnt8_0==sys_NMI_dcnt8_1){	// GPS N/A
			GPS_UPDATE_FLAG=0;
			edit_sys_time();
		}else{
			if(sys_NMI_dcnt8_0>sys_NMI_dcnt8_1){ // NMEA
				GPS_UPDATE_FLAG &= ~TSIP_mask;
				edit_NMEA_time();
			}else{	// TSIP
				GPS_UPDATE_FLAG &=  TSIP_mask;
				edit_TSIP_time();
			};
		};
	};
}

